@@ -663,6 +663,7 @@ characters are then removed.
 
 sub _slash_hack {
    for ( my $spec = shift ) {
+      confess "undef arg" unless defined $spec ;
       s{[/\\]+}{/}g ;
       s{^/}{}g ;
       s{/\Z}{}g ;
@@ -916,9 +917,9 @@ sub DESTROY {
 
 Copyright 2000, Perforce Software, Inc.  All Rights Reserved.
 
-This will be licensed under a suitable license at a future date.  Until
-then, you may only use this for evaluation purposes.  Besides which, it's
-in an early alpha state, so you shouldn't depend on it anyway.
+This module and the VCP package are licensed according to the terms given in
+the file LICENSE accompanying this distribution, a copy of which is included in
+L<vcp>.
 
 =head1 AUTHOR
 
